"use strict"

if (!net)
  var net = {}
  
if (!net.tschmid)
  net.tschmid = {};
  
if (!net.tschmid.sieve)
  net.tschmid.sieve = {};

if (!net.tschmid.sieve.editor)
  net.tschmid.sieve.editor = {};

if (!net.tschmid.sieve.editor.text)
  net.tschmid.sieve.editor.text = {};  
  
// We need two global variables for backward compatiblity...
// they may be removed as soon as the editor communication is migrated to post message
var editor = null;
var onActiveLineChange = null;


/**
 * Glues two frame via html5 post message.
 **/
  
(function() {

  var hlLine = null;
  var listener = {}
  
  var glue = new net.tschmid.sieve.iframe.glue();  	
  glue.setListener(function(event, data) { listener.onMessage(event, data) });  
		
  
  function winHeight() {
    return window.innerHeight || (document.documentElement || document.body).clientHeight;
  }
  
  function setFullScreen(cm) {
    var wrap = cm.getWrapperElement();
    wrap.className += " CodeMirror-fullscreen";
    wrap.style.height = winHeight() + "px";
    document.documentElement.style.overflow = "hidden";
    cm.refresh();
  }        
    
  // This function is called externaly upon reloading and updating scripts  
  onActiveLineChange = function() 
  {
    var cur = editor.getLineHandle(editor.getCursor().line);
        
    if (cur == hlLine)
      return;
    	
    editor.removeLineClass(hlLine, "background", "activeline");
    hlLine = editor.addLineClass(cur, "background", "activeline");
  }    	  

  function onChange() {
  	glue.sendMessage("onChange");
  }

  function findString(token, isCaseSensitive, isReverse) { 
  	
    // ... convert to lowercase, if the search is not case sensitive...
  
    function maxCursor(start,end)
    {
      if (start.line > end.line)
        return start

      if (start.line < end.line)
        return end;
    
      // start.line == end.line
      if (start.ch > end.ch)
        return start;
      
      return end;
    }
  
    function minCursor(start,end)
    {
      if (start.line <end.line)
        return start

      if (start.line > end.line)
        return end;
    
      // start.line == end.line
      if (start.ch > end.ch)
        return end;
      
      return start;
    }
  
    var start = editor.getCursor(true);
    var end = editor.getCursor(false);
  
    var cursor = editor.getSearchCursor(
      token,
      isReverse ? minCursor(start,end) : maxCursor(start,end),
      !isCaseSensitive);
  
    if (!cursor.find(isReverse))
    { 
      // warp search at top or bottom
      cursor = editor.getSearchCursor(
        token,
        isReverse ? {line: editor.lineCount() - 1} : {line: 0, ch: 0},
        !isCaseSensitive);
    
      if (!cursor.find(isReverse))
        return;
    }
  
    if (isReverse)
      editor.setSelection(cursor.from(), cursor.to());
    else
      editor.setSelection(cursor.to(), cursor.from());

    glue.sendMessage("onStringFound", true);  
    return;
  }
  
  function replaceString(oldToken, newToken, isCaseSensitive, isReverse) {  	
  	
    if (isCaseSensitive) 
    {
      if (editor.getSelection() != oldToken)
        findString(oldToken, isSensitive, isReverse);
      
      if (editor.getSelection() != oldToken)
        return;
    }
    
    if (!isCaseSensitive)
    {
      if (editor.getSelection().toLowerCase() != oldToken.toLowerCase())
        findString(oldToken, isCaseSensitive, isReverse);
      
      if (editor.getSelection().toLowerCase() != oldToken.toLowerCase())
        return;
    }

    editor.replaceSelection(newToken);    

    onChange();
  }

  	 	 
  listener.onMessage = function(event, data) {  
  	
  	if (event == "focus") {
      editor.focus();			
  	}
  	  	
  	// Load a new script. It will discard the current script
  	// the history and the cursorposition are reset to defaults.
  	if (event == "loadScript") {
  	  editor.setValue(data);
      editor.setCursor({line:0,ch:0});
      editor.clearHistory();
      
      // ensure the active line cursor changed...
      onActiveLineChange();
      
      glue.sendMessage("onScriptLoaded");
        
      return;
  	}
  	  
  	// Updates the current script. The history as well as 
  	// the cursor position is maintained.
	if (event == "setScript") { 
	  editor.setValue(data);
	  return;
	}
		
	if (event == "getScript") {
	  	
	  // Get the current script...
	  var script = editor.getValue();
	  // ... and ensure the line endings are sanatized
	  script = script.replace(/\r\n|\r|\n|\u0085|\u000C|\u2028|\u2029/g,"\r\n");
	  	
	  glue.sendMessage("onGetScript", script);	    
	  return;
	}
	  
	if (event == "replaceSelection") {
	  editor.replaceSelection(data);
	  return;
	}
	  
	if (event == "selectAll") {
	  editor.setSelection({line:0,ch:0},{line: editor.lineCount() - 1});
	  return;
	}
	  
	if (event == "undo") {
	  editor.undo();
	  return;
	}
	  
	if (event == "redo") {
	  editor.redo();
	  return;
	}
	  
	if (event == "replaceString") {	
	  replaceString(data.oldToken, data.newToken, data.isSensitive, data.isReverse);
	  return;
	}
	  
	if (event == "findString") {
	  findString(data.token, data.isCaseSensitive, data.isReverse);
	  return;
	}	
  };

  // Export the constructor...
  net.tschmid.sieve.editor.text.init = function() {
  	
    CodeMirror.on(window, "resize", function() {
      document.body.getElementsByClassName("CodeMirror-fullscreen")[0]
        .CodeMirror.getWrapperElement().style.height = winHeight() + "px";
    });
    
    editor = CodeMirror.fromTextArea(document.getElementById("code"), {
      lineNumbers: true,
      theme: "eclipse",
      matchBrackets: true
    });
  
    setFullScreen(editor,true);
    
    hlLine = editor.addLineClass(0, "background", "activeline");
  
    editor.on("cursorActivity", function() { onActiveLineChange(); });        
    editor.on("change", function() { onChange(); });    
  };
  
}());
  